--	1
--	Use CROSS APPLY to bring student info
--	along with their Facility fines
select *
from	Students as ST
cross apply (
		select	  FF.STUDENT_ID
				, FF.AMOUNT
		from	FACILITY_FINES as FF
		where	FF.STUDENT_ID = ST.STUDENT_ID) as FFines

--	2
--	Use CROSS APPLY to bring student info
--	along with their Parking fines
select *
from	Students as ST
cross apply (
		select	PF.AMOUNT
		from	PARKING_FINES as PF
		where	PF.STUDENT_ID = ST.STUDENT_ID ) as PFines		

--	3		
--	Use CROSS APPLY display student information
--	and the sum of their parking fines		
select *
from	Students as ST
cross apply (
		select	  sum(PF.AMOUNT) as AMOUNT
		from	PARKING_FINES as PF
		where	PF.STUDENT_ID = ST.STUDENT_ID
		group by PF.STUDENT_ID) as PFines

--	4
--	Cross Apply vs inner join and group by		
--	to get aggregate
Select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, sum(PF.AMOUNT) as AMOUNT
from	Students as ST
		inner join	PARKING_FINES as PF	
			on ST.STUDENT_ID = PF.STUDENT_ID
GROUP BY ST.STUDENT_ID, ST.NAME, ST.PHONE

--	5
--	How about a CTE with Cross Apply?
;with ParkingFinesCTE(STUDENT_ID, AMOUNT, RANKING)
as 
(
	select	  STUDENT_ID
			, sum(PF.AMOUNT) as AMOUNT
			, RANK() over (order by sum(PF.AMOUNT) desc) as RANKING 
	from	PARKING_FINES as PF	
	group by STUDENT_ID
)
SELECT  ST.*
		, CTE.AMOUNT
FROM    ParkingFinesCTE as CTE
cross apply Students as ST	
where	ST.STUDENT_ID = CTE.STUDENT_ID

--	Lets look at the plans when we create and use a function
--	in our CROSS APPLY query

--	6
--	All fines using CROSS APPLY with function 
--	VS Inner join
--	Display all fines using Cross Apply with function
select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, APF.AMOUNT
from	Students as ST
		cross apply fn_AllParkingFines(ST.STUDENT_ID) as APF
		
--	Get all fines using Inner join 
Select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, PF.AMOUNT
from	Students as ST
		inner join	PARKING_FINES as PF	
			on ST.STUDENT_ID = PF.STUDENT_ID		

--	7				
--	SUM of fines using Cross apply with Function vs INNER JOIN

--	SUM of fines with function and CROSS APPLY 
select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, PF.AMOUNT
from	Students as ST
		cross apply fn_ParkingFines(ST.STUDENT_ID) as PF

--	Use INNER JOIN and group by to get the SUM of fines
Select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, sum(PF.AMOUNT) as AMOUNT
from	Students as ST
		inner join	PARKING_FINES as PF	
			on ST.STUDENT_ID = PF.STUDENT_ID
GROUP BY ST.STUDENT_ID, ST.NAME, ST.PHONE


/****** Outer Apply ******/

--	8
--	Outer Apply
--	Use outer apply to bring student info
--	along with their Facility fines, if they have any
select *
from	Students as ST
outer apply (
		select	  FF.STUDENT_ID
				, FF.AMOUNT
		from	FACILITY_FINES as FF
		where	FF.STUDENT_ID = ST.STUDENT_ID) as FFines
		
--	9
--	Use outer apply to bring student info
--	along with their Parking fines, if they have any
select *
from	Students as ST
outer apply (
		select	  PF.STUDENT_ID
				, PF.AMOUNT
		from	PARKING_FINES as PF
		where	PF.STUDENT_ID = ST.STUDENT_ID
		) as PFines		

--	10
--	Use outer apply display student information
--	and the sum of their parking fines, if they have one		
select *
from	Students as ST
OUTER apply (
		select	  sum(PF.AMOUNT) as AMOUNT
		from	PARKING_FINES as PF
		where	PF.STUDENT_ID = ST.STUDENT_ID
		group by PF.STUDENT_ID) as PFines


--	11
--	OUTER APPLY vs LEFT OUTER JOIN 
--	Use OUTER APPLY to bring student info
--	along with their Parking fines, if they have any
select *
from	Students as ST
outer apply (
		select	  PF.STUDENT_ID
				, PF.AMOUNT
		from	PARKING_FINES as PF
		where	PF.STUDENT_ID = ST.STUDENT_ID
		) as PFines		

--	Use left outer join to display all parking fines
select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, PF.AMOUNT
from	Students as ST
		left outer join PARKING_FINES as PF
			on ST.STUDENT_ID = PF.STUDENT_ID

--	12
--	OUTER APPLY vs LEFT OUTER JOIN with SUM
--	Use outer apply to bring student info
--	along with the sum of their Parking fines
--	if they have any
select *
from	Students as ST
OUTER apply (
		select	  sum(PF.AMOUNT) as AMOUNT
		from	PARKING_FINES as PF
		where	PF.STUDENT_ID = ST.STUDENT_ID
		group by PF.STUDENT_ID) as PFines	

--	Sum the data using LEFT OUTER JOIN
select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, sum(PF.AMOUNT) as AMOUNT
from	Students as ST
		left outer join PARKING_FINES as PF
			on ST.STUDENT_ID = PF.STUDENT_ID
group by ST.STUDENT_ID, ST.NAME, ST.PHONE

--	13
--	All fines using OUTER APPLY with function 
--	VS LEFT OUTER JOIN
--	Display all fines using Cross Apply with function
--	Use the function
select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, APF.AMOUNT
from	Students as ST
		OUTER apply fn_AllParkingFines(ST.STUDENT_ID) as APF	

--	Use left outer join to display all parking fines
select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, PF.AMOUNT
from	Students as ST
		left outer join PARKING_FINES as PF
			on ST.STUDENT_ID = PF.STUDENT_ID

--	14
--	SUM of fines using OUTER APPLY with Function 
--	vs LEFT OUTER JOIN and group by
--	SUM of fines with function and OUTER APPLY 
select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, PF.AMOUNT
from	Students as ST
		outer apply fn_ParkingFines(ST.STUDENT_ID) as PF

--	Use LEFT OUTER JOIN and group by to get the SUM of fines
Select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, sum(PF.AMOUNT) as AMOUNT
from	Students as ST
		left outer join	PARKING_FINES as PF	
			on ST.STUDENT_ID = PF.STUDENT_ID
GROUP BY ST.STUDENT_ID, ST.NAME, ST.PHONE

--	15
--	Find the rows on the left where results are null on the right		
select	  ST.STUDENT_ID
		, ST.NAME
		, ST.PHONE
		, APF.AMOUNT
from	Students as ST
		OUTER apply fn_AllParkingFines(ST.STUDENT_ID) as APF	
where	APF.AMOUNT is null

